#!/bin/bash

# Tests the core auction flow

# import utilities
HERE=$(cd $(dirname $BASH_SOURCE) && pwd)
source $HERE/testutil.sh

colineard tx colinearcore unlock-all-funds -y --from alice \
| swallow "Unlock all funds before proceeding"

colineard tx colinearcore new-auction asdf asdf 200 uusdc $(now + 3500) \
    -y --from bob \
    | expect_fail "Can't create auction below min lease period" 

colineard tx colinearcore new-auction asdf asdf 200 uusdc $(now + 8200000) \
    -y --from bob \
    | expect_fail "Can't create auction above max lease period" 

before=$(get_balance $BOB uusdc)

colineard tx colinearcore new-auction asdf asdf 200 uusdc $(now + 3700) \
    -y --from bob \
    | expect_success "New auction is created" 

get_balance $BOB uusdc | expect_change -200 $before "Change in Bob's balance"

colineard tx colinearcore new-bid $(get_last_auction_index) 100 \
    -y --from alice \
    | expect_fail "Can't bid without locking CLR"

colineard tx colinearcore lock-funds 550000000 -y --from alice \
    | expect_success "Can lock above the maximum (550 CLR)"

colineard tx colinearcore lock-funds 500000000 -y --from alice \
    | expect_success "Lock funds before bidding"

colineard tx colinearcore new-bid $(get_last_auction_index) 100 \
    -y --from alice \
    | expect_success "New bid is correctly created"

colineard tx colinearcore new-bid $(get_last_auction_index) 90 \
    -y --from alice \
    | expect_success "Can bid below price ceiling"

colineard tx colinearcore new-bid $(get_last_auction_index) 100 \
    -y --from alice \
    | expect_fail "Can't create duplicate bids"

colineard tx colinearcore new-bid $(get_next_auction_index) 100 \
    -y --from alice \
    | expect_fail "Can't create bids for nonexistent auctions"

colineard tx colinearcore new-bid $(get_last_auction_index) 600 \
    -y --from alice \
    | expect_fail "Can't bid over price ceiling"

colineard tx colinearcore new-bid $(get_last_auction_index) 0 \
    -y --from alice \
    | expect_fail "Can't bid 0 uusdcs"

colineard q colinearcore auction-bids $(get_last_auction_index) \
    | jq -M ".bids | length" \
    | assert_eq 2 "Number of auction bids"

before=$(get_balance $BOB uusdc)

# Was experiencing issues with bob's balance not updating, probably
# because we weren't waiting long enough
log_info "Waiting 12s until auction expires..."
sleep 12

colineard tx colinearcore new-bid $(get_last_auction_index) 50 \
    -y --from alice \
    | expect_fail "Can't add bids after auction expiry"

colineard q colinearcore show-auction $(get_last_auction_index) \
    | jq -rM ".auction.best.amount" \
    | assert_eq 90 "Remaining vested amount from finalized top bid"

get_balance $BOB uusdc | expect_change 110 $before "Change in Bob's balance"
